/*
---------------------------------------------------------------------------
 $Revision: 1.1.2.21.2.1 $                                                                 
 $Date: 2015/05/11 17:40:32 $  

 NAME: sea_data_validation

 TYPE: macro

 DESCRIPTION:
   This macro does the following:
   - Verifies existence of submission tables
   - Verifies format of submission tables
   - Checks to see which files have been validated
   - Calls DQ macros
   - Creates submission file DQ report
   
 SAS VERSION:
 9.4
   
 PARAMETERS: 
   +-----------------------------------------------------------------
   | Input Item Name: libref
   | Description: library reference where error tables will be placed
   | Value:  Valid SAS libref
   +-----------------------------------------------------------------
   | Input Item Name: DTTMFMT
   | Description:  Datetime format
   | Required: Yes
   | Value: Default=NLDATM21., valid SAS datetime format
   +-----------------------------------------------------------------
   | Input Item Name: TMFMT
   | Description:  Time format
   | Required: Yes
   | Value: Default=NLTIMAP11., valid SAS time format
   +-----------------------------------------------------------------
   | Input Item Name: DTFMT
   | Required: Yes
   | Description:  LIBREF associated with source data table
   | Value: default=NLDATE10., valid SAS date format
   +-----------------------------------------------------------------
   | Input Item Name: VALIDATION_RUN_ID
   | Description:  Unique identifier for set of submission tables
   | Required: Yes     
   +-----------------------------------------------------------------
   | Input Item Name: RESTART
   | Description:  To restart validation or validate only unvalidated
   |               files
   | Required: Yes 

 REQUIREMENT: Macro is run within a SAS session that is connected to the OMR
              For example - SAS workspace server or SAS stored process server

 HISTORY:
 userid yyyymmdd defectid desc    
 cabahl 20140719          Initial 
 cabahl 20140827 DE4667   Updated SEA_PROVIDER_TARGETS check
 cabahl 20140918 DE4711   Set DQT to 0 as default value
 cabahl 20140929 DE4785   Fix typo 
 cabahl 20140930 DE4791   Check multiple SEA_PROF file naming
 cabahl 20141030 DE4791   Move check for multiple SEA_PROF files after update
                          of SEA_CHECK_SUM
 cabahl 20141113 DE4991   Fixed typos in natural key section of data quality
                          report
 cabahl 20141205 DE5123   updated member and provider id length to 32
 cabahl 20141211 DE5157   upcase fields names in all submission data sets
 cabahl 20140505 S1176519 corrected mis-spelling of Minimum in column label
                          data quality report completed successful converted to
                          completed successfully
-----------------------------------------------------------------------------
 Copyright (c) 2014, SAS Institute Inc., Cary, NC, USA, All Rights Reserved 
------------------------------------------------------------------------------
*/

%macro sea_data_validation(LIBREF=
                          ,DTTMFMT=NLDATM21. 
                          ,TMFMT=NLTIMAP11. 
                          ,DTFMT=NLDATE10.   
                          ,VALIDATION_RUN_ID=
                          ,RESTART=N);

   %global RETCODE LIBRAW SEA_DQ_VALIDATION_STATUS;
   %let retcode=0;  
   %let sferr=0; 
   
   %if %sysfunc(libref(RAW)) > 0 %then %do;
     libname raw "&SourceFiles"; 
   %end;

   %let libraw=RAW;

   %if &libref=%str()  %then %let libref=RAW;
   %if &restart=%str() %then %let restart=N;

   /*************************************CLEANUP*************************************************/
  
   /********************************************************************
   * Delete ERR_* tables if they exist
   ********************************************************************/
   proc sql noprint;
     select memname 
       into :errtabs separated by ' '
       from dictionary.tables
      where libname=upcase("&libraw")
        and memname like 'ERR%';

     select memname 
       into :worktabs separated by ' '
       from dictionary.tables
      where libname=upcase("WORK")
        and memname like 'SEA%';
   quit;

   proc datasets lib=&libraw nolist nowarn;
    %if %symexist(ERRTABS) %then %do; 
      delete &errtabs;
      run;
    %end;
    /* Start over - delete RAW.SEA_CHECK_SUM */
    %if &RESTART=Y %then %do;
      delete SEA_CHECK_SUM;
      run;    
    %end;
    /* Delete SEA_PROF if it is a view */
    %if %sysfunc(exist(&libraw..SEA_PROF,VIEW)) > 0 %then %do;
      delete SEA_PROF(mt=VIEW);
      run;        
    %end;
   quit;

   %if %symexist(WORKTABS) %then %do;
     proc datasets lib=WORK nolist;
       delete &worktabs;
       run;
     quit;
   %end;

   /*************************************CLEANUP FINISHED****************************************/
   
   /**************************************SEA_PROF Naming Check *********************************/
   proc sql;
    /* Get SEA_PROF files -naming check */
   create table spchk as
     select memname
       from dictionary.tables
      where libname=upcase("&libraw")
        and memname like 'SEA_PROF%'
      order by memname
    ;    
  quit;
  
  data null_;
    set spchk end=eof nobs=obs;
    by memname;
    retain issue;
    if obs=1 then stop; 
    n=_n_;
    num=input(substr(memname,9),8.);
    if num ne _n_ then issue='1';
    if eof then do;
      if issue='1' then call symput('sea_prof_suffix_issue',issue);
    end;  
  run;    
      
  %if %symexist(SEA_PROF_SUFFIX_ISSUE) %then %do;
    %let retcode=1;
    %let SEA_DQ_VALIDATION_STATUS=U;
    %let sferr=1;
    %let sf_errmsg1=SEA_PROF files are not numbered sequentially;
    %GOTO ERROR_END;
  %end;  

   /*************************************TEMPLATE CREATION **************************************/

   /*********************************************************************
   *   Create work version of submission tables with 0 records.  The
   *   variable able is used to indicate whether it is required or not.
   *********************************************************************/ 

   data WORK.SEA_ENROLL (label='R');
     attrib CONSISTENT_MEMBER_ID  length=$32  label='Required=Y';
     attrib ENROLLMENT_BEGIN_DT   length=8  FORMAT=&DTFMT INFORMAT=&DTFMT 
                                  label='Required=Y';
     attrib ENROLLMENT_END_DT     length=8  FORMAT=&DTFMT INFORMAT=&DTFMT 
                                  label='Required=Y';
    stop; 
   run; 

   data WORK.SEA_MEMBER (label='R');
     attrib CONSISTENT_MEMBER_ID  length=$32     label='Required=Y'; 
     attrib GENDER_CD             length=$1      label='Required=Y'; 
     attrib RACE_CD               length=$1      label='Required=N'; 
     attrib MEMBER_ZIP_CD         length=$5      label='Required=N'; 
     attrib YEAR_OF_BIRTH_NO      length=4       label='Required=Y'; 
     attrib USER_DEFINED_PROVIDER_ID length=$32  label='Required=N';
     attrib PATIENT_PCP_ID        length=$32     label='Required=N';
     attrib MEMBER_DEATH_DT       length=8 FORMAT=&DTFMT INFORMAT=&DTFMT 
                                                 label='Required=N';
    stop; 
   run; 

   data WORK.SEA_PROF (label='R');
     attrib CONSISTENT_MEMBER_ID  length=$32  label='Required=Y';
     attrib CLAIM_ID              length=$32  label='Required=Y';
     attrib PROVIDER_ID           length=$32  label='Required=N';
     attrib CLAIM_LINE_NO         length=8  label='Required=Y';
     attrib ALLOWED_AMT           length=8  FORMAT=NLNUM14.2 INFORMAT=NLNUM14.2 
                                  label='Required=N';
     attrib STANDARD_PAYMENT_AMT  length=8  FORMAT=NLNUM14.2 INFORMAT=NLNUM14.2 
                                  label='Required=N';
     attrib FACILITY_TYPE_CD      length=$4  label='Required=Y';
     attrib SERVICE_BEGIN_DT      length=8  FORMAT=&DTFMT INFORMAT=&DTFMT label='Required=Y';
     attrib SERVICE_END_DT        length=8  FORMAT=&DTFMT INFORMAT=&DTFMT label='Required=Y';
     attrib PLACE_OF_SVC_CD       length=$2  label='Required=Y';
     attrib CLAIM_TYPE_CD         length=$2  label='Required=Y';
     attrib DIAG_VOCAB_CD         length=$10  label='Required=N';
     attrib PRINCIPAL_DIAG_CD     length=$20  label='Required=N';
     attrib SECONDARY_DIAG1_CD    length=$20  label='Required=N';
     attrib PROC_VOCAB_CD         length=$10  label='Required=N';
     attrib PRINCIPAL_PROC_CD     length=$20  label='Required=N';
     attrib PRINCIPAL_PROC_MOD_CD length=$20  label='Required=N';
     attrib SECONDARY_PROC1_CD    length=$20  label='Required=N';
    stop; 
   run; 

   data WORK.SEA_PROVIDERS (label='R');
     attrib PROVIDER_ID           length=$32  label='Required=Y';
     attrib PROVIDER_GROUP_NM     length=$10  label='Required=N';
     attrib PROVIDER_SYSTEM_NM    length=$10  label='Required=N';
     attrib PROVIDER_TAX_ID       length=$10  label='Required=N';
     attrib PROVIDER_SPECIALTY_1_CD  length=$2  label='Required=N';
     attrib PROVIDER_SPECIALTY_2_CD  length=$2  label='Required=N';
     attrib PROVIDER_SPECIALTY_3_CD  length=$2  label='Required=N';
     attrib PROVIDER_ZIP_CD       length=$5  label='Required=N';
     attrib PROVIDER_PILOT_NM     length=$10  label='Required=N';
     attrib PROVIDER_ACO_NM       length=$10  label='Required=N';
    stop; 
   run; 

   data WORK.SEA_PROVIDER_TARGETS (label='O');
     attrib PROVIDER_ID           length=$32  label='Required=Y';
     attrib CONDITION             length=$6   label='Required=Y';
     attrib PAC_PCT               length=8    label='Required=Y';    
     attrib MARGIN_PCT            length=8    label='Required=Y';
     attrib SRF_PCT               length=8    label='Required=Y';
     stop; 
   run; 

   data WORK.SEA_RX (label='R');
     attrib CONSISTENT_MEMBER_ID  length=$32  label='Required=Y';
     attrib ALLOWED_AMT           length=8  FORMAT=NLNUM14.2 INFORMAT=NLNUM14.2 
                                  label='Required=Y';
     attrib DRUG_VOCAB_CD         length=$10  label='Required=Y';
     attrib DRUG_CD               length=$20  label='Required=Y';
     attrib SUPPLY_DAY_NO         length=8  label='Required=Y';
     attrib PRESCRIPTION_FILLED_DT  length=8  FORMAT=&DTFMT INFORMAT=&DTFMT 
                                  label='Required=Y';
     stop;
   run; 

   data WORK.SEA_STAY (label='R');
     attrib CONSISTENT_MEMBER_ID  length=$32  label='Required=Y';
     attrib CLAIM_ID              length=$32  label='Required=Y';
     attrib PROVIDER_ID           length=$32  label='Required=N';
     attrib ALLOWED_AMT           length=8  FORMAT=NLNUM14.2 INFORMAT=NLNUM14.2 
                                  label='Required=N';
     attrib STANDARD_PAYMENT_AMT  length=8  FORMAT=NLNUM14.2 INFORMAT=NLNUM14.2 
                                  label='Required=N';
     attrib FACILITY_TYPE_CD      length=$4  label='Required=Y';
     attrib SERVICE_BEGIN_DT      length=8  FORMAT=&DTFMT INFORMAT=&DTFMT label='Required=Y';
     attrib SERVICE_END_DT        length=8  FORMAT=&DTFMT INFORMAT=&DTFMT label='Required=Y';
     attrib ADMISSION_SOURCE_CD   length=$1  label='Required=N';
     attrib ADMIT_TYPE_CD         length=8  label='Required=Y';
     attrib DISCHARGE_STATUS_CD   length=$2  label='Required=Y';
     attrib PLACE_OF_SVC_CD       length=$2  label='Required=Y';
     attrib CLAIM_TYPE_CD         length=$2  label='Required=Y';
     attrib DIAG_VOCAB_CD         length=$10  label='Required=N';
     attrib PRINCIPAL_DIAG_CD     length=$20  label='Required=N';
     attrib SECONDARY_DIAG1_CD    length=$20  label='Required=N';
     attrib PROC_VOCAB_CD         length=$10  label='Required=N';
     attrib PRINCIPAL_PROC_CD     length=$20  label='Required=N';
     attrib PRINCIPAL_PROC_MOD_CD  length=$20  label='Required=N';
     attrib SECONDARY_PROC1_CD    length=$20  label='Required=N';
     attrib REV_CD1               length=$20  label='Required=N';
     attrib MS_DRG_CD             length=$20  label='Required=Y';
     attrib APR_DRG_CD            length=$20  label='Required=N';
    stop; 
  run; 
  /*****************************END TEMPLATE CREATION*********************************************/

  /***************************************************************************
  * DE5157 Need to upcase all of the variable names within the submission data
  *        sets
  ***************************************************************************/
   %let cols=0;
   proc sql noprint;
     create table cols as
       select libname,
              memname,
              name
         from dictionary.columns
        where libname = upcase("&libraw")
          and memname like 'SEA%'
          and memname ne 'SEA_CHECK_SUM'
          and name ne upcase(name)
        order by memname, name;
    
     select nobs
       into :cols
       from dictionary.tables
       where libname="WORK"
         and memname="COLS";
   quit;

  %if &cols > 0 %then %do;
    data _null_;
      set cols;
      by memname name;
      retain tabs cols 0;
      if first.memname then do;
        tabs=tabs+1;
        cols=0;
        call symput(compress('tab'||put(tabs,best.)),memname);
      end;
      cols=cols+1;
      
      if last.memname then do;
         call symput('tabs',put(tabs,best.));
         call symput(compress('cols'||put(tabs,best.)),put(cols,best.));
      end;
      
      tcn=compress(put(tabs,best.)||put(cols,best.));
      call symput(compress('rn_'||tcn),compress(name)||'='||compress(upcase(name)));
    run;

    proc datasets lib=&libraw nolist;
     %do t=1 %to &tabs;
       modify &&tab&t;
       rename 
         %do c=1 %to &&cols&t;
            %let cc=&t.&c;
            &&rn_&cc
         %end;
        ;
       run;
     %end;
    quit;
  %end;

  /***************************************************************************
  * Table and Column Existence Checks
  ***************************************************************************/ 
  proc sql;
    /* Submission file attributes */
    create table submission_tables as
     select memname as subfile,
            compress(memname,'0123456789') as table,
            nobs as observations,
            case
              when compress='NO' then '0'
              else '1'
            end as compress,
            memlabel as valchk
       from dictionary.tables
      where libname=upcase("&libraw")
        and memname like 'SEA%'
        and memname ne 'SEA_CHECK_SUM'
      order by memname
    ;

    create table sea_check_sum as
      select "&validation_run_id" length=8 format=$8. as validation_run_id
            ,a.memname as table
            ,b.subfile
            ,compress(compress) length=1 as compress
            ,compress(required) length=1 as required
            ,valchk
            ,observations
        from (select memname, memlabel as required
                from dictionary.tables
               where libname=upcase("WORK") 
                 and memname like 'SEA%'
                 and memname not in ('SEA_CHECK_SUM' 'SUBMISSION_TABLES')) a left join
             submission_tables b
          on a.memname=b.table   
       order by a.memname;

     /* Submission table column attributes */
     create table template as
       select distinct memname as table,
              compress(name) as col,
              type,
              length,
              format,
              case 
                when scan(label,1,'=')='Required' then scan(label,2,'=')
                else ''
               end as field_required,
              required as table_required,
              observations as table_obs
         from dictionary.columns a,
              sea_check_sum b
        where libname = upcase("WORK")
          and a.memname = b.table
        order by table,col;

     create table raw as
       select memname as subfile,
              compress(memname,'0123456789') as table,              
              upcase(name) as col,
              type as raw_type,
              length as raw_length,
              format as raw_format
         from dictionary.columns
        where libname = upcase("&libraw")
          and memname in (select subfile from sea_check_sum)
        order by table, col;
   quit;

   /* Column attribute checks */
   %let errchk=0;
   data &libref..err_table_columns
        cchk(keep=table vcnt rawvcnt)
     ;
     merge template (in=a)
           raw      (in=b)
           end=eof
     ;
     by table col;
     retain vcnt rawvcnt; 

     if first.table then do;
       vcnt=0;
       rawvcnt=0;
     end;

     if a then do;
       vcnt=vcnt+1;

       if not b then do;
         missing=1;
       end;
       else do;
         missing=0;
         rawvcnt=rawvcnt+1;
       end;

       if format = "&DTFMT" then type='date';
       if raw_format in ('DATE9.' 'MMDDYY10.') then raw_type='date'; 

       if type ne raw_type then type_check=1;
       else type_check=0;

       if raw_length < length then length_check=1;
       else length_check=0;

       if table_required='R' or 
         (table_required='O' and table_obs > 0) then do; 
          if type_check=1 or length_check=1 or missing=1 then do;
            output &libref..err_table_columns;
            call symput('errchk','1');
          end;
       end;
     end;
     if last.table then output cchk;
   run;

   /***************************************************************************************** 
   *  Create the SEA_CHECK_SUM data set if does not exist
   *****************************************************************************************/
   %if %sysfunc(exist(&libraw..sea_check_sum))=0 %then %do;
     data &libraw..sea_check_sum(drop=missing_tables no_obs);
       attrib validation_run_id length=$8   label='Validation Run Identifier'
              table             length=$32  label='Submission Table'
              subfile           length=$32  label='Submitted File'
              required          length=$1   label='R=Required,O=Optional'
              expected_columns  length=8    label='Number of Expected Columns'
              columns           length=8    label='Number of Columns in Submitted Table'
              observations      length=8    label='Number of Records in Submitted Table'
              min_dt            length=8    format=&DTFMT label='Minimum Date'
              max_dt            length=8    format=&DTFMT label='Maximum Date'
              total_claims      length=8    label='Total Claims'
              total_errors      length=8    label='Total Number of Errors'
              total_dups        length=8    label='Total Number of Duplicate Records'
              total_warnings    length=8    label='Total Number of Warnings'
              compress          length=$8   label='0=Not Compressed,1=Compressed'
              validated         length=$8   label='0=Not Validated,1=Validated'
              validation_dttm   format=&DTTMFMT label='Table Validation Completed'
              process_dttm      format=&DTTMFMT label='Datetime when validation process started.'
              run_order         length=8    label='Table Run order within data validation'
              ;
       merge sea_check_sum(drop=valchk)
             cchk(rename=(vcnt=expected_columns
                          rawvcnt=columns))
              end=eof;
       by table;
       process_dttm=datetime();

       select(table);
          when('SEA_MEMBER') run_order=1;
          when('SEA_PROVIDERS') run_order=2;
          when('SEA_ENROLL') run_order=4;
          when('SEA_PROVIDER_TARGETS') run_order=3;
          when('SEA_STAY') run_order=5;
          when('SEA_RX') run_order=6;
          when('SEA_PROF') run_order=7;
          otherwise;
       end;
       
       validated='0';
       validation_dttm =.;

       /* Determine if any table is missing */
       length missing_tables no_obs $500;
       retain missing_tables no_obs;

       /*
       * Stay and RX can have 0 observations
       */
       if required='R' and observations=. then 
         missing_tables=strip(missing_tables)||' '||compress(table);
       if required='R' and observations=0 then 
         no_obs=strip(no_obs)||' '||compress(table);
         
       if eof then do;
         call symput('missing_tables',strip(missing_tables));
         call symput('no_obs',strip(no_obs));
       end;
     run;
   %end;

   /***************************************************************************************** 
   *  SEA_CHECK_SUM exists
   *****************************************************************************************/
   %else %do;
      proc sort data=&libraw..sea_check_sum;
        by table;
      run;

      data &libraw..sea_check_sum(drop=missing_tables unequal_obs no_obs 
                                       existing_obs);
        merge &libraw..sea_check_sum (in=a drop=required
                                      rename=(observations=existing_obs))
              sea_check_sum(in=b keep=table observations required)
              cchk(in=c)
              end=eof;
        by table;
        length missing_tables no_obs unequal_obs $500;

        if required='R' and observations=. then 
          missing_tables=strip(missing_tables)||' '||compress(table);

        /*
        * Stay and RX can have 0 observations
        */
        if required='R' and observations=0 then 
          no_obs=strip(no_obs)||' '||compress(table);

        /*
        * Verify that a file has been validated
        */
        if validated='1' then do;
          if observations ne existing_obs then do;
            unequal_obs=strip(unequal_obs)||' '||compress(table);
            validated='0';          
          end;

          if put(validated_dttm,&dttmfmt) ne valchk then validated='0';

        end;

        if eof then do;
         call symput('missing_tables',strip(missing_tables));
         call symput('no_obs',strip(no_obs));
         if unequal_obs ne '' then
           call symput('unequal_obs',strip(unequal_obs));
        end;

      run;

   %end;

   %if &missing_tables ne %str() %then %do;
     %let retcode=1; 
     %let errmsg_1=ERROR: The following submission tables are missing: &missing_tables;
     %GOTO VALIDATION_REPORTS;
   %end;

   %if &errchk=1 %then %do;
      %let retcode=1; 
      %let errmsg_2=ERROR: Columns are missing or are not correctly formated check &libref..err_table_columns;
      %GOTO VALIDATION_REPORTS;
   %end;

   %if &no_obs ne %str() %then %do;
     %let retcode=1; 
     %let errmsg_3=ERROR: The following submission tables are have 0 records: &no_obs;
     %GOTO VALIDATION_REPORTS;
   %end; 

   %if %symexist(unequal_obs)=1 %then %do;
     %let retcode=2;
     %let errmsg_4=WARNING: Submission table record count does not match SEA_CHECK_SUM revalidating - &unequal_obs;
   %end;

   %TABLE_VALIDATION:
   %if &retcode = 0 %then %do;
 
      proc sort data=&libraw..sea_check_sum out=val;
        where validated='0' and observations > 0;
        by run_order subfile;
      run;

      %let dqt=0;
      data _null_;
        set val end=eof;
        by run_order subfile;
        retain n 0;
        length errtable tablast $32;

        if required='R' or (required='O' and observations not in (.,0)) then do;
          n+1;
          /*submitted file name*/
          call symput(compress('tab'||put(n,best.)),compress(subfile));
          /*template submissiontable name */
          call symput(compress('tabx'||put(n,best.)),compress(table));
          /* get indication of last PROF submitted file */
          if last.run_order and last.subfile and subfile ne table then do;
             call symput(compress('tablast'||put(n,best.)),
                                            compress(tranwrd(table,'SEA_','')));
          end;
          else call symput(compress('tablast'||put(n,best.)),''); 

          errtable=tranwrd(subfile,'SEA','ERR');
          call symput(compress('etab'||put(n,best.)),compress(errtable));
          macro=compress('%SEA_DQ'||tranwrd(table,'SEA',''));
          call symput(compress('dqmacr'||put(n,best.)),macro);
        end;
        if eof then call symput('dqt',put(n,best.));
      run;

      %let member=&libraw..SEA_MEMBER;
      %let provider=&libraw..SEA_PROVIDERS;

      %if &dqt > 0 %then %do;

         %do i = 1 %to &dqt;
           %put Submission Table - &&tab&i; 
           %put Running Data Quality Macro - &&dqmacr&i;
 
           data required_fields(keep=stmt col type);
             set template end=eof;
             where table="&&tabx&i" and field_required='Y';
             length stmt $100;
             if lowcase(type)='char' then stmt=strip(col)||"=''";
             else stmt=strip(col)||"=.";
           run;

           &&dqmacr&i(indsn=&libraw..&&tab&i
                     ,outdsn=&libref..&&etab&i
                   );

           %if &retcode=1 %then %do;
              %let errmsg_6=ERROR: &&tab&i had a major data quality issue.;
           %end;

           %if &retcode=0 %then %do; 
              /* No issues - table validated */
              data &libraw..sea_check_sum;
                set &libraw..sea_check_sum;
                if subfile="&&tab&i" then do;
                  validated='1';
                  validation_dttm=datetime();
                  call symput('valdttm',compress(put(validation_dttm,&dttmfmt)));

                %if %symexist(&&tabx&i.._MINDT) %then %do;
                  min_dt=input(symget("&&tabx&i.._MINDT"),8.);  
                  max_dt=input(symget("&&tabx&i.._MAXDT"),8.); 
                %end;  

                %if %symexist(&&tabx&i.._CLAIMS) %then %do;
                  total_claims=input(compress(symget("&&tabx&i.._CLAIMS")),8.);
                %end;

                  total_errors=input(compress(symget("&&tabx&i.._ERRORS")),8.);
                  total_dups=input(compress(symget("&&tabx&i.._DUPRECS")),8.); 

                %if %symexist(&&tabx&i.._WARNINGS) %then %do;
                  total_warnings=input(compress(symget("&&tabx&i.._WARNINGS")),8.);
                %end;                  
                end;
              run;

              proc datasets lib=&libraw nolist;
                modify &&tab&i (label="&valdttm");
                run;
              quit; 

           %end;
           %else %do;
              %let sferr=%eval(&sferr + 1);
              %let sf_errmsg&sferr=Table, &&tab&i, had data quality issues. Check the Data Quality Issues report.; 

              /* Errors and Duplicates */
              data &libraw..sea_check_sum;
                set &libraw..sea_check_sum;
                if subfile="&&tab&i" then do;
                %if %symexist(&&tabx&i.._MINDT) %then %do;
                  min_dt=input(symget("&&tabx&i.._MINDT"),8.);  
                  max_dt=input(symget("&&tabx&i.._MAXDT"),8.); 
                %end;  

                %if %symexist(&&tabx&i.._CLAIMS) %then %do;
                  total_claims=input(compress(symget("&&tabx&i.._CLAIMS")),8.);
                %end;

                  total_errors=input(compress(symget("&&tabx&i.._ERRORS")),8.);
                  total_dups=input(compress(symget("&&tabx&i.._DUPRECS")),8.); 
                  
                %if %symexist(&&tabx&i.._WARNINGS) %then %do;
                  total_warnings=input(compress(symget("&&tabx&i.._WARNINGS")),8.);
                %end;                  
                  
                end;

              run;
           %end; 
           
           /* Multiple file SEA_PROF duplicate check */
           %let SEA_PROF_DUPRECS_B=0;
           %if &&tablast&i = PROF %then %do;

             proc sql noprint;
               select ("&libraw.."||subfile)
                 into :files separated by ' '
                 from val
                 where table='SEA_PROF';
             quit; 
 
             data &libraw..SEA_PROF / view=&libraw..SEA_PROF;
                set &files;
                by consistent_member_id claim_id claim_line_no;
             run;

             data _null_;
               set &libraw..SEA_PROF end=eof;
               by consistent_member_id claim_id claim_line_no;

               retain duprecs 0;
               /* duplicate claim lines */
               if first.claim_line_no then do;
               end;
               else do;
                 duprecs=duprecs+1;
                 record_no=_n_;
                 field="CLAIM_ID CLAIM_LINE_NO";
                 dq_check='Duplicate Record';
                 reason=compress(consistent_member_id||'^'||claim_id||'^'||
                        claim_line_no);
                 if duprecs > 0 then do;
                   call symput('SEA_PROF_DUPRECS_B',put(duprecs,best.));
                   stop;
                 end;
                end;

                if eof then call symput('SEA_PROF_DUPRECS_B',put(duprecs,best.));
             run;
             %put &sea_prof_duprecs_b;

             /* Duplicate prof records found when multiple files sent */
             %if &sea_prof_duprecs_b>0 %then %do;
               %let retcode=1;
               %let errmsg=The SEA_PROF files when combined have duplicate records.;
               
               data &libraw..sea_check_sum;
                  set &libraw..sea_check_sum; 
                  if table = 'SEA_PROF' then do;
                    total_dups=input(compress(symget("SEA_PROF_DUPRECS_B")),8.);
                    validate="0";                  
                  end;        
               run; 
             %end;
             %else %let retcode=0;
 
           %end;            
         %end; 
      %end;
   %end;

   %VALIDATION_REPORTS:
    
   %sea_utl_report_ods_set_up (sea_rpt_type=&RPTTYPE,
                               sea_rpt_file_name=data_quality,
                               sea_rpt_file_title=&validation_run_id Data Quality Report); 

   footnote "Run on %sysfunc(TRIM(%QSYSFUNC(DATE(),NLDATE20.))) at %sysfunc(TRIM(%SYSFUNC(TIME(),TIMEAMPM12.)))";

   %if %symexist(FILESETNAME)=0 %then %let FILESETNAME=;

   title1 "Data Validation Results for &validation_run_id &filesetname";

   /* Submission data overview */
   title2 "Submission Files Overview";
   title3 "Validation Status";

   proc sort data=&libraw..sea_check_sum;
     by table subfile;
   run;

   data val_rpt(drop=validated);
     set &libraw..sea_check_sum;
     by table subfile;
     if required='O' and observations < 1 then delete;
     length val_status $35 compress1 $3;
     label val_status='Validation Status'
           required='Table Required'
           mis_col='Missing Columns'
           observations='Total Records'
           total_errors='Total Errors'
           total_dups='Total Duplicate Records'
           compress1='Compressed';
     mis_col=expected_columns-columns;
     
     if validated='0' then 
        val_status='Completed Unsuccessfully';
     if validated='-1' then
        val_status='Combined files have dup records'; 
     if validated='1' then 
        val_status='Completed Successfully'; 
        
     if compress='1' then compress1='Yes';
     else compress1='No';
     
     if total_errors=. then total_errors=0;
     if total_dups=. then total_dups=0;
     if total_warnings=. then total_warnings=0;
   run;

   proc print data=val_rpt noobs label width=UBY;                
     by table;
     var subfile required mis_col total_errors total_dups total_warnings val_status;
   run;

   title3 "Data Profile";
   proc print data=val_rpt noobs label width=UBY;
     by table; 
     var subfile observations total_claims min_dt max_dt compress1;
   run;

   data natkeys;
    length table $32 natkey $100;
    table='SEA_ENROLL';
    natkey='CONSISTENT_MEMBER_ID ENROLLMENT_BEGIN_DT ENROLLMENT_END_DT';
    output;
    table='SEA_MEMBER';
    natkey='CONSISTENT_MEMBER_ID';
    output;
    table='SEA_PROVIDERS';
    natkey='PROVIDER_ID';
    output;
    table='SEA_PROVIDER_TARGETS';
    natkey='PROVIDER_ID CONDITION';
    output;
    table='SEA_PROF';
    natkey='CONSISTENT_MEMBER_ID CLAIM_ID CLAIM_LINE_NO';
    output;
    table='SEA_RX';
    natkey='CONSISTENT_MEMBER_ID DRUG_CD PRESCRIPTION_FILLED_DT SUPPLY_DAY_NO ALLOWED_AMT';
    output;
    table='SEA_STAY';
    natkey='CONSISTENT_MEMBER_ID CLAIM_ID SERVICE_BEGIN_DT SERVICE_END_DT';
    output;

    label table ='Table'
          natkey='Natural Key';
  run;

  title3 'Natural Keys';
  proc report data=natkeys;
    column table natkey;
    define table / flow;
    define natkey / flow;
  run;
  
   /*
     Need to format data for better report layout
   */
   %if &errchk > 0 %then %do;
     title2 "Data Quality Issues";
     title3 "Submission Column Format Incorrect or Column Missing";
     proc print data=&libref..err_table_columns noobs label split='*' uniform;
       label table = 'Submission Table'
             col = 'Field' 
             type='Expected*Type'
             length='Expect*Length'
             format='Expected*Format'
             raw_type='Actual*Type'
             raw_length='Actual*Length'
             raw_format='Actual*Format'
       ;      
       by table;
       var col type length format missing raw_type raw_length raw_format;
     run;
   %end;
   %else %do;
     /* No column issues drop error table */
     proc sql;
       drop table &libref..err_table_columns;
     quit;
   %end;

   /* Error Summary */
   %let etabs=0;
   proc sql noprint; 
     create table error_tables as   
       select memname
         from dictionary.tables
        where libname=upcase("&libref")
          and nobs > 0
          and memname like 'ERR_%'
      ;

      select count(*) 
        into :etabs
        from error_tables;
   quit;

   %if &ETABS > 0 %then %do;

      %do e = 1 %to &etabs; 
         %let x_error=;
         %let x_warn=; 
 
         data _null_;
           set error_tables;
           if _n_=&e then do;
              call symput('error_table',compress(memname));
              call symput('etab',compress(tranwrd(memname,'ERR','SEA')));
           end;
         run;

         proc sql noprint;
           select distinct(type) 
             into :x_error
             from &libref..&error_table
            where type='E';

           select distinct(type) 
             into :x_warn
             from &libref..&error_table
            where type='W';
         quit;

       %if &x_error=E %then %do;
         title2 "Summary of Data Quality Issues in &etab";
         title3 "Data Errors";
         proc sql;
           create table e1 as
           select dq_check,
                  field,
                  count(*) label='Occurrence' as freq 
             from &libref..&error_table 
            where type='E'
            group by dq_check, field;
         quit;

         proc print data=e1 noobs label;
         run;
       %end;

       %if &x_warn=W %then %do;         
         title2 "Summary of Data Quality Issues in &etab";
         title3 "Data Warnings";
         proc sql;
           create table w1 as
           select dq_check,
                  field,
                  count(*) label='Occurrence' as freq 
             from &libref..&error_table 
            where type='W'
            group by dq_check, field;
         quit;

         proc print data=w1 noobs label;
         run;
       %end;
      %end;
   %end;     

   /* Close report */
   ods &RPTTYPE close;
   ods listing; 

   title1;
   title2;
   footnote;

   /* Determine Validation Status */
   %let unvalidated=0;
   data _null_;
     set &libraw..sea_check_sum end=eof;
     where validated='0';
     retain unvalidated 0;
     if required='R' then unvalidated=unvalidated+1;
     else do;
       if observations > 0 then unvalidated=unvalidated+1;
     end;    

     if eof then call symput('unvalidated',put(unvalidated,best.)); 
   run;

   %if &errchk=1 %then %do;
     %if &unvalidated > 0 %then %let SEA_DQ_VALIDATION_STATUS=CU;
     %else %let SEA_DQ_VALIDATION_STATUS=CV;
   %end;
   %else %do;
     %if &unvalidated > 0 %then %let SEA_DQ_VALIDATION_STATUS=U;
     %else %let SEA_DQ_VALIDATION_STATUS=V;
   %end;   

   %ERROR_END:
     %if &retcode > 0 %then %do;
       %do i=1 %to 6;
         %if %symexist(ERRMSG_&i)=1 %then %do;
           %put &&errmsg_&i;
         %end;
       %end;

       %if &sferr > 0 %then %do;
         %do s=1 %to &sferr;
           %put ERROR: &&sf_errmsg&s;      
         %end;
       %end;
     %end;
   
%mend;



 
