/*
---------------------------------------------------------------------------
 $Revision: 1.1.2.20.2.1 $                                                                 
 $Date: 2015/05/11 17:40:31 $  

 NAME: sea_dq_stay

 TYPE: macro

 DESCRIPTION:
   This macro performs the data quality checks on SEA_STAY table.
  
 SAS VERSION:
 9.4
   
 PARAMETERS: 
 
 REQUIREMENT: 

 HISTORY:
 userid yyyymmdd defectid desc    
 cabahl 20140719          Initial 
 cabahl 20140812          Updated some of the tests from error to warning
 cabahl 20141105 DE4791   warnings need to be sent to data_validation and 
                          retcode set to 3 
 cabahl 20141205 DE5123   updated provider and member id length to 32 
 cabahl 20150505 S1176528 overlapping SEA_STAY added to the error output data
                          set
-----------------------------------------------------------------------------
 Copyright (c) 2014, SAS Institute Inc., Cary, NC, USA, All Rights Reserved 
------------------------------------------------------------------------------
*/

%macro sea_dq_stay(indsn=
                  ,outdsn= 
                  );

  /* 
  * Create global macro variables 
  */
  %global sea_stay_duprecs sea_stay_errors sea_stay_claims
          sea_stay_mindt sea_stay_maxdt sea_stay_warnings    
  ;

  %let sea_stay_duprecs=0;
  %let sea_stay_errors=0;
  %let sea_stay_mindt=;
  %let sea_stay_maxdt=;
  %let sea_stay_claims=0;
  %let retcode=0;

  %if &indsn=%str() %then %do;

  %end;

  %if &outdsn=%str() %then %do;

  %end;  

  %let inlib=%scan(&indsn,1,.);

  %sea_data_val_formats;

  /* get required variables to test for missing values */
  %let rq=0;
  data _null_;
    set required_fields end=eof;
    call symput(compress('rq_'||put(_n_,best.)),strip(stmt));
    call symput(compress('rqf_'||put(_n_,best.)),strip(col));
    if eof then call symput('rq',put(_n_,best.));
  run;

  %if &rq=0 %then %do;
    %let retcode=1;
    %let errmsg_enroll=ERROR: Required fields missing;
    %GOTO DQ_STAY_END;
  %end;  

  %let index1=0;
  %let index2=0;
  proc sql noprint;
    select '1' as index
      into :index1
      from dictionary.indexes
     where libname=upcase("&inlib")
       and memname=upcase("%scan(&indsn,2,.)")
       and upcase(indxname) = 'STAY1';

    select '1' as index
      into :index2
      from dictionary.indexes
     where libname=upcase("&inlib")
       and memname=upcase("%scan(&indsn,2,.)")
       and upcase(indxname) = 'STAY2';
  quit;

  /* Add index to SEA_STAY */
  proc datasets lib=&inlib nolist nowarn;
    modify %scan(&indsn,2,.);
    %if &index1=0 %then %do;
       index create stay1=(consistent_member_id claim_id service_begin_dt service_end_dt); 
    %end;
    %if &index2=0 %then %do;
       index create stay2=(consistent_member_id service_begin_dt service_end_dt); 
    %end;
    run;
  quit;
  
  data work.stay_members;
    attrib consistent_member_id length=$32;
    stop;
  run;

  data work.stay_providers;
    attrib provider_id length=$32;
    stop;
  run;

  proc sql;
    alter table work.stay_members
       add constraint consistent_member_id primary key (consistent_member_id)
    ;
    alter table work.stay_providers
       add constraint provider_id primary key (provider_id)
    ; 
  quit;

  data &outdsn (keep=field dq_check reason type consistent_member_id 
                     claim_id service_begin_dt service_end_dt)
       work.stay_members 
       work.stay_providers
      ;
      attrib record_no length=8    label='Record Number'
             field     length=$32  label='Field with DQ Issue'
             dq_check  length=$100 label='Data Quality Check'
             reason    length=$100 label='Field Value with DQ Issue'
             type      length=$1      
             ;
      set &indsn end=eof;
      by consistent_member_id claim_id service_begin_dt service_end_dt;
      retain errors duprecs warnings min_stay_dt max_stay_dt claims 0;

      type='E';

      /* Output unique member ids */
      modify work.stay_members key=consistent_member_id / unique;
      if _iorc_ eq %sysrc(_DSENOM) then do;
        output stay_members; 
        _error_=0;
      end;   

      /* Output unique provider ids */
      if provider_id ne '' then do;
        modify work.stay_providers key=provider_id / unique;
        if _iorc_ eq %sysrc(_DSENOM) then do;
          output stay_providers; 
          _error_=0;
        end;   
      end;

      /* Min and Max service dates */
      if min_stay_dt=0 then min_stay_dt=service_begin_dt;
      if service_begin_dt < min_stay_dt then min_stay_dt=service_begin_dt;
      if service_end_dt > max_stay_dt   then max_stay_dt=service_end_dt;

      /* duplicate claim lines */
      if first.service_end_dt then do;
      end;
      else do;
         duprecs=duprecs+1;
         record_no=_n_;
         field="Natural Key";
         dq_check='Duplicate Record';
         reason=compress(consistent_member_id||'^'||claim_id||'^'||
                put(service_begin_dt,nldate.)||'^'||put(service_end_dt,nldate.));
         output &outdsn; 
      end;

      /* duplicate claims */
      if first.claim_id then do;
        claims=claims+1;
      end;
      else do;
         duprecs=duprecs+1;
         record_no=_n_;
         field="CLAIM_ID";
         dq_check='Duplicate Claim Record';
         reason=compress(consistent_member_id||'^'||claim_id);
         output &outdsn; 
      end;

      /* required fields - missing values */
      %do rz=1 %to &rq;
        if &&rq_&rz then do;
          errors=errors+1;
          record_no=_n_;
          field="&&rqf_&rz";
          dq_check="Field not populated.";
          reason="No data";
          output &outdsn; 
        end;
      %end;

      /* Illogical service dates */
      if service_begin_dt > service_end_dt then do;
         errors=errors+1;
         record_no=_n_;
         field="Service_Begin_DT Service_End_DT";
         dq_check="Service Begin Date > Service End Date";
         reason=put(service_begin_dt,NLDATE.)||'^'||
               put(service_end_dt,NLDATE.);
         output &outdsn; 
      end;

      /* Place of Service */
      if put(place_of_svc_cd,$placesvc.)='' then do;
         warnings=warnings+1;
         record_no=_n_;
         field="PLACE_OF_SVC_CD";
         dq_check="Place of service code not found in list of accepted codes.";
         reason=place_of_svc_cd;
         type='W';
         output &outdsn; 
      end;

      /* Claim Type */
      if claim_type_cd ne '' then do;
        if claim_type_cd ne 'IP' then do;
          errors=errors+1;
          record_no=_n_;
          field="CLAIM_TYPE_CD";
          dq_check="Claim type code value not IP.";
          reason=claim_type_cd;
          output &outdsn; 
        end;
      end;

      /* Facility Type */
      if put(facility_type_cd,$facility.)='' then do;
         warnings=warnings+1;
         record_no=_n_;
         field="FACILITY_TYPE_CD";
         dq_check="Facility type code not found in list of accepted codes.";
         reason=facility_type_cd;
         type='W';
         output &outdsn; 
      end;

      /* Diagnosis/Procedure Code Checks */
      /* both principal codes are blank */
      if ktrim(kleft(principal_diag_cd)) = '' and 
         ktrim(kleft(principal_proc_cd)) = '' then do;
         errors=errors+1;
         record_no=_n_;
         field="PRINCIPAL_DIAG_CD";
         dq_check="Both Principal_Diag_CD and Principal_Proc_CD were not populated.";
         reason='No Value';
         output &outdsn; 
      end;

      /* principal_diag_cd not blank and diag_vocab_cd blank */
      if ktrim(kleft(principal_diag_cd)) ne '' 
        and ktrim(kleft(diag_vocab_cd)) = '' then do;
         errors=errors+1;
         record_no=_n_;
         field="DIAG_VOCAB_CD";
         dq_check="Principal_Diag_Cd is populated and Diag_Vocab_CD was not populated.";
         reason='No Value';
         output &outdsn; 
      end;
      
      /* principal_proc_cd not blank and proc_vocab_cd blank */      
      if ktrim(kleft(principal_proc_cd)) ne '' 
        and ktrim(kleft(proc_vocab_cd)) = '' then do;
         errors=errors+1;
         record_no=_n_;
         field="PROC_VOCAB_CD";
         dq_check="Principal_Proc_Cd is populated and Proc_Vocab_CD was not populated.";
         reason='No Value';
         output &outdsn; 
      end;      

      /* diag_vocab_cd not populated correctly */
      if ktrim(kleft(diag_vocab_cd)) ne '' and 
         put(diag_vocab_cd,$diagvocab.)='' then do;
         errors=errors+1;
         record_no=_n_;
         field="DIAG_VOCAB_CD";
         dq_check="Diagnosis vocabulary code not found in list of accepted codes.";
         reason=diag_vocab_cd;
         output &outdsn; 
      end;

      /* proc_vocab_cd not populated correctly */
      if ktrim(kleft(proc_vocab_cd)) ne '' and 
         put(proc_vocab_cd,$procvocab.)='' then do;
         errors=errors+1;
         record_no=_n_;
         field="PROC_VOCAB_CD";
         dq_check="Procedure vocabulary code not found in list of accepted codes.";
         reason=proc_vocab_cd;
         output &outdsn; 
      end;

      /* Claim amount check */
      if allowed_amt=. and standard_payment_amt=. then do;
         errors=errors+1;
         record_no=_n_;
         field="ALLOWED_AMT STANDARD_PAYMENT_AMT";
         dq_check="Claims amount, Allowed or Standard Payment amount columns, not populated.";
         reason='No value';
         output &outdsn; 
      end;

      /* Provider ID Missing */
      if provider_id='' then do;
         warnings=warnings+1;
         record_no=_n_;
         field="PROVIDER_ID";
         dq_check="Provider_ID was not populated.";
         reason='No value';
         type='W';
         output &outdsn; 
      end;

      /* Admit Type */
      if put(admit_type_cd,admittype.)='' then do;
         warnings=warnings+1;
         record_no=_n_;
         field="ADMIT_TYPE_CD";
         dq_check="Admit type code not found in list of accepted codes.";
         reason=admit_type_cd;
         type='W';
         output &outdsn; 
      end;

      /* Discharge Status */
      if put(discharge_status_cd,$discharge.)='' then do;
         warnings=warnings+1;
         record_no=_n_;
         field="DISCHARGE_STATUS_CD";
         dq_check="Discharge status code not found in list of accepted codes.";
         reason=discharge_status_cd;
         type='W';       
         output &outdsn; 
      end;

      if eof then do;
        call symput('sea_stay_warnings',put(warnings,best.));
        call symput('sea_stay_errors',put(errors,best.));
        call symput('sea_stay_duprecs',put(duprecs,best.));
        call symput('sea_stay_mindt',put(min_stay_dt,8.));
        call symput('sea_stay_maxdt',put(max_stay_dt,8.));
        call symput('sea_stay_claims',put(claims,best.));
      end;
    run;
    
    /* Overlapping service dates */
    data work.overlap(keep=record_no field dq_check reason type);
      attrib record_no length=8    label='Record Number'
             field     length=$32  label='Field with DQ Issue'
             dq_check  length=$100  label='Data Quality Check'
             reason    length=$100 label='Field Value with DQ Issue'
             type      length=$1
             ;
      set &indsn end=eof;
      by consistent_member_id service_begin_dt service_end_dt;
      retain duprecs errors 0;

      /* duplicate claims  */
      if first.service_end_dt then do;
      end;
      else do;
         duprecs=duprecs+1;
         record_no=_n_;
         field="Member Id and Service Dates";
         dq_check='Duplicate service begin and end dates';
         reason=compress(consistent_member_id||'^'||claim_id||'^'||
                put(service_begin_dt,nldate.)||'^'||put(service_end_dt,nldate.));
         type='E';       
         output; 
      end;

      prev_service_end_dt=lag(service_end_dt);
      if not first.consistent_member_id then do; 
        if service_begin_dt < prev_service_end_dt then do;          
          record_no=_n_;
          field="Service Dates";
          dq_check="Service date overlap";
          reason='Service start date within the date range of another claim.';
          type='W';
          output;           
        end;
      end;

      if eof then do;
        call symput('stay_errors',put(errors,best.));
        call symput('stay_duprecs',put(duprecs,best.));
      end;
    run;

    /* Member in SEA_STAY not in SEA_MEMBER */
    data work.err_sm (keep=record_no field dq_check reason type);
      attrib record_no length=8    label='Record Number'
             field     length=$32  label='Field with DQ Issue'
             dq_check  length=$100  label='Data Quality Check'
             reason    length=$100 label='Field Value with DQ Issue'
             type      length=$1
             ;
      merge &member(in=a)
            work.stay_members(in=b)
            end=eof;
      by consistent_member_id;

      retain errors 0;

      if b and not a then do;
         errors=errors+1;
         record_no=_n_;
         field=consistent_member_id;
         dq_check='Consistent_Member_Id value not found in SEA_MEMBER';
         reason=consistent_member_id;
         type='E';
         output;       
       end;

       if eof then do;
         call symput('sea_stay_memerrs',put(errors,best.));
       end;
    run;

    /* Provider in SEA_STAY and not SEA_PROVIDERS */ 
    proc sort data=stay_providers;
      by provider_id;
    run;

    data work.err_sp (keep=record_no field dq_check reason type);
      attrib record_no length=8    label='Record Number'
             field     length=$32  label='Field with DQ Issue'
             dq_check  length=$100  label='Data Quality Check'
             reason    length=$100 label='Field Value with DQ Issue'
             type      length=$1
             ;
      merge &provider(in=a)
            work.stay_providers(in=b)
            end=eof;
      by provider_id;

      retain errors 0;

      if b and not a then do;
         errors=errors+1;
         record_no=_n_;
         field=provider_id;
         dq_check='Provider_Id value not found in SEA_PROVIDERS';
         reason=provider_id;
         type='E';
         output;       
       end;

       if eof then do;
         call symput('sea_stay_proverrs',put(errors,best.));
       end;
    run;

    %if &sea_stay_memerrs > 0  or 
        &sea_stay_proverrs > 0 or
        &sea_stay_errors > 0 or
        &stay_errors > 0 %then %do;
      %let retcode=2;
      %let sea_stay_errors=%eval(&sea_stay_errors + &sea_stay_memerrs + &sea_stay_proverrs + &stay_errors);
    %end;

    %if &sea_stay_duprecs > 0 
        or &stay_duprecs > 0 %then %do;
      %let sea_stay_duprecs=%eval(&sea_stay_duprecs + &stay_duprecs); 
      %let retcode=1;
    %end;   

    data &outdsn;
      set &outdsn
          overlap
          err_sm
          err_sp;     
    run; 
    
  %DQ_STAY_END:
    %if &retcode=0 and &sea_stay_warnings = 0 %then %do;
      proc sql;
        drop table &outdsn;
      quit; 
    %end;    
%mend;
